import { AppContainer, AppEventType, HistoryEventType } from '@propulse/app';
import React, { useEffect } from 'react';
import { useDispatch, useStore } from 'react-redux';
import { historyLocationChange } from '../actions';
import { appStateKey } from '../index';
import { rehydrate as rehydrateApp } from '../rehydrate';
function addHistoryListener(app, key, dispatch) {
    return app.histories[key].subscribe(event => {
        switch (event.type) {
            case HistoryEventType.LocationChange:
                dispatch(historyLocationChange(key, event.action, event.index, event.entries, event.length, event.location));
                break;
            case HistoryEventType.LocationStateChange:
                dispatch(historyLocationChange(key, app.histories[key].action, app.histories[key].index, app.histories[key].entries, app.histories[key].length, app.histories[key].location));
                break;
        }
    });
}
export const ConnectedAppContainer = ({ app, children, rehydrate }) => {
    const dispatch = useDispatch();
    useEffect(() => {
        const unregisterCallbacks = [];
        const historiesUnregisterCallbacks = {};
        if (app) {
            for (const key in app.histories) {
                if (app.histories.hasOwnProperty(key)) {
                    historiesUnregisterCallbacks[key] = addHistoryListener(app, key, dispatch);
                }
            }
            unregisterCallbacks.push(app.subscribe(event => {
                switch (event.type) {
                    case AppEventType.HistoryAdd:
                        historiesUnregisterCallbacks[event.key] = addHistoryListener(app, event.key, dispatch);
                        break;
                    case AppEventType.HistoryRemove:
                        if (historiesUnregisterCallbacks[event.key]) {
                            historiesUnregisterCallbacks[event.key]();
                        }
                        break;
                }
            }));
        }
        return () => {
            for (const key in historiesUnregisterCallbacks) {
                if (historiesUnregisterCallbacks.hasOwnProperty(key) &&
                    historiesUnregisterCallbacks[key]) {
                    historiesUnregisterCallbacks[key]();
                }
            }
            unregisterCallbacks.forEach(c => c());
        };
    }, [app, dispatch]);
    const store = useStore();
    if (app && rehydrate) {
        const state = store.getState();
        if (state[appStateKey]) {
            rehydrateApp(app, state[appStateKey]);
        }
    }
    return <AppContainer app={app} children={children}/>;
};
